@@ -26,7 +26,7 @@ QVariant SavedQuickSettingsModel::data(const QModelIndex &index, int role) const
 
 int SavedQuickSettingsModel::rowCount(const QModelIndex &parent) const
 {
-    Q_UNUSED(parent);
+    Q_UNUSED(parent)
     return m_data.count();
 }
 
