@@ -31,6 +31,9 @@ class FolioApplication;
 class FolioApplicationFolder : public QObject, public std::enable_shared_from_this<FolioApplicationFolder>
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_UNCREATABLE("")
+
     Q_PROPERTY(QString name READ name WRITE setName NOTIFY nameChanged)
     Q_PROPERTY(QList<FolioApplication *> appPreviews READ appPreviews NOTIFY applicationsChanged)
     Q_PROPERTY(ApplicationFolderModel *applications READ applications NOTIFY applicationsReset)
@@ -38,7 +41,7 @@ class FolioApplicationFolder : public QObject, public std::enable_shared_from_th
 public:
     typedef std::shared_ptr<FolioApplicationFolder> Ptr;
 
-    FolioApplicationFolder(HomeScreen *parent = nullptr, QString name = QString{});
+    FolioApplicationFolder(HomeScreen *homeScreen = nullptr, QString name = QString{}, QObject *parent = nullptr);
 
     static std::shared_ptr<FolioApplicationFolder> fromJson(QJsonObject &obj, HomeScreen *parent);
     QJsonObject toJson() const;
